# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.__internal__.tracking namespace.
"""

import sys as _sys

from tensorflow.python.training.saving.saveable_object_util import saveable_objects_from_trackable
from tensorflow.python.training.tracking.autotrackable import AutoTrackable
from tensorflow.python.training.tracking.base import CheckpointInitialValue
from tensorflow.python.training.tracking.base import CheckpointInitialValueCallable
from tensorflow.python.training.tracking.base import Trackable
from tensorflow.python.training.tracking.base import TrackableReference
from tensorflow.python.training.tracking.base import no_automatic_dependency_tracking
from tensorflow.python.training.tracking.base_delegate import DelegatingTrackableMixin
from tensorflow.python.training.tracking.data_structures import TrackableDataStructure
from tensorflow.python.training.tracking.data_structures import sticky_attribute_assignment
from tensorflow.python.training.tracking.data_structures import wrap_or_unwrap as wrap
from tensorflow.python.training.tracking.graph_view import ObjectGraphView
from tensorflow.python.training.tracking.util import TrackableSaver
from tensorflow.python.training.tracking.util import register_session_provider
from tensorflow.python.training.tracking.util import streaming_restore